import pytest
from selenium.webdriver.common.by import By

from pages import Page


class TestWebElementWrapperMethods:

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_tag_name_of_element(self, driver, logger):
        """example of get tag name of element"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkABTest = (By.CSS_SELECTOR, "[href='/abtest']")
        logger.info(f"Tag name => {page.find_element(*lnkABTest).tag_name}")
        logger.info(f"Tag name => {page.tag_name(*lnkABTest)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_text_of_element(self, driver, logger):
        """example of text of element"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkABTest = (By.CSS_SELECTOR, "[href='/abtest']")
        logger.info(f"Link Text => {page.find_element(*lnkABTest).text}")
        logger.info(f"Link Text => {page.text(*lnkABTest)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_click_method(self, driver, logger):
        """example of click method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkABTest = (By.CSS_SELECTOR, "[href='/abtest']")
        page.find_element(*lnkABTest).click()
        page.wait_for_a_while(2)

        page.back()
        page.wait_for_a_while(2)
        page.click(*lnkABTest)
        page.wait_for_a_while(2)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_click_and_wait_method(self, driver, logger):
        """example of click and wait method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkABTest = (By.CSS_SELECTOR, "[href='/abtest']")
        page.click_and_wait(*lnkABTest, 5)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_element_to_be_clickable(self, driver, logger):
        """example of element_to_be_clickable method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkABTest = (By.CSS_SELECTOR, "[href='/abtest']")
        page.element_to_be_clickable(*lnkABTest)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_submit(self, driver, logger):
        """example of submit form method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        page.click(*lnkLogin)

        txtUsername = (By.NAME, "username")
        page.send_keys(*txtUsername, "tomsmith")

        txtPassword = (By.NAME, "password")
        page.send_keys(*txtPassword, "SuperSecretPassword!")

        btnLogin = (By.CSS_SELECTOR, "button.radius")
        page.submit(*btnLogin)

        page.wait_for_a_while(4)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_is_displayed(self, driver, logger):
        """example of is_displayed method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"Is login link displayed? A. {page.is_displayed(*lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_location_once_scrolled_into_view(self, driver, logger):
        """example of location_once_scrolled_into_view method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")
        page.wait_for_a_while(2)

        lnkWYSIWYG = (By.CSS_SELECTOR, "[href='/tinymce']")
        page.location_once_scrolled_into_view(*lnkWYSIWYG)

        page.wait_for_a_while(2)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_size(self, driver, logger):
        """example of size method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"Size of element = {page.size(*lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_value_of_css_property(self, driver, logger):
        """example of value_of_css_property method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"Size of element = {page.value_of_css_property('color', *lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_location_(self, driver, logger):
        """example of location method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"location of element = {page.location(*lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_rect_(self, driver, logger):
        """example of rect method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"rect of element = {page.rect(*lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_aria_role(self, driver, logger):
        """example of aria_role method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"aria_role of element = {page.aria_role(*lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_accessible_name(self, driver, logger):
        """example of accessible_name method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        logger.info(f"accessible_name of element = {page.accessible_name(*lnkLogin)}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_screenshot_as_base64(self, driver, logger):
        """example of screenshot_as_base64 method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        screenshot = page.screenshot_as_base64(*lnkLogin)

        from nrobo.util.common import Common
        Common.save_base64string(screenshot, 'element_screenshot.png')

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_screenshot_as_png(self, driver, logger):
        """example of screenshot_as_png method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        screenshot_as_binary_data = page.screenshot_as_png(*lnkLogin)

        from nrobo.util.common import Common
        Common.save_bytes_to_file(screenshot_as_binary_data, 'element_screenshot_as_binary.png')

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_screenshot_as_file(self, driver, logger):
        """example of screenshot method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkLogin = (By.CSS_SELECTOR, "[href='/login']")
        page.screenshot('element_screenshot_as_file.png', *lnkLogin)







"""
@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import pytest
from pages.PagePyPiHome import PagePyPiHome


class TestPyPiHomePage():

    @pytest.mark.sanity
    def test_google_branding_displayed_on_home_page(self, driver, logger):
        # Instantiate page object
        page_pypi_home = PagePyPiHome(driver, logger)
        # call page method
        page_pypi_home.open()

        # Asset test condition
        assert page_pypi_home.search_button_present() == True




import sys

import allure
import pytest


class Examples:

    @pytest.mark.sanity
    def test_this_is_marked_as_sanity_test(self, driver, logger):
        logger.info("This is a sanity test.")
        # Your sanity test logic will go below
        # ...
        # ...

    @pytest.mark.regression
    def test_this_is_marked_as_regression_test(self, driver, logger):
        logger.info("This is a regression test.")
        # Your regression test logic will go below
        # ...
        # ...

    @pytest.mark.ui
    def test_this_is_marked_as_ui_test(self, driver, logger):
        logger.info("This is a ui test.")
        # Your ui test logic will go below
        # ...
        # ...

    @pytest.mark.unit
    def test_this_is_marked_as_unit_test(self, driver, logger):
        logger.info("This is a unit test.")
        # Your unit test logic will go below
        # ...
        # ...

    @pytest.mark.nogui
    def test_this_is_marked_as_nogui_test(self, driver, logger):
        logger.info("This is a nogui test.")
        # Your gui test logic will go below
        # ...
        # ...

    @pytest.mark.api
    def test_this_is_marked_as_api_test(self, driver, logger):
        logger.info("This is a api test.")
        # Your api test logic will go below
        # ...
        # ...

    @pytest.mark.flaky(reruns=3, reruns_delay=2)
    def test_this_is_marked_as_flaky_test(self, driver, logger):
        logger.info("This is a flaky test. Test that may fail for the first time. If fails, run thrice.")
        # Your flaky test logic will go below
        # ...
        # ...

    @allure.title("Test Authentication")
    @allure.description(
        "This test attempts to log into the website using a login and a password. Fails if any error happens.\n\nNote "
        "that this test does not test 2-Factor Authentication.")
    @allure.description_html("This is an example of <bold>html</b> description.")
    @allure.tag("NewUI", "Essentials", "Authentication")
    @allure.severity(allure.severity_level.CRITICAL)
    @allure.label("owner", "John Doe")
    @allure.link("https://dev.example.com/", name="Website")
    @allure.issue("AUTH-123")
    @allure.testcase("TMS-456")
    def test_this_is_an_example_of_allure_description_links_and_other_metadata(self, driver, logger):
        logger.info("This is an example of allure description, links and other metadata")

    @pytest.mark.filterwarnings("ignore:api v1")
    def test_filter_warnings(self, driver, logger):
        logger.info("This is an example of filter warnings.")

    @pytest.mark.skip(reason="Reason for skipping this test")
    def test_skip(self, driver, logger):
        logger.info(f"skip the given test function with an optional reason. "
                    f"Example: skip(reason='no way of currently testing this') skips the test.")

    @pytest.mark.skipif(sys.platform == 'win32', reason="This is not intended to run on windows machine.")
    def test_skip_if_condition_is_met(self, driver, logger):
        logger.info("skip the given test function if any of the conditions evaluate to True. Example: "
                    "skipif(sys.platform == 'win32') skips the test if we are on the win32 platform.")

    @pytest.mark.xfail(sys.platform == 'win32', reason="This test will fail on windows.", raises=NotImplemented, run=False, strict=False)
    def test_expected_failure_if_condition_is_met(self, driver, logger):
        logger.info("""
        Parameters:
            condition (Union[bool, str]) – Condition for marking the test function as xfail (True/False or a condition string). If a bool, you also have to specify reason (see condition string).
            
            reason (str) – Reason why the test function is marked as xfail.
            
            raises (Type[Exception]) – Exception class (or tuple of classes) expected to be raised by the test function; other exceptions will fail the test. Note that subclasses of the classes passed will also result in a match (similar to how the except statement works).
            
            run (bool) – Whether the test function should actually be executed. If False, the function will always xfail and will not be executed (useful if a function is segfaulting).
            
            strict (bool) –
            
            If False the function will be shown in the terminal output as xfailed if it fails and as xpass if it passes. In both cases this will not cause the test suite to fail as a whole. This is particularly useful to mark flaky tests (tests that fail at random) to be tackled later.
            
            If True, the function will be shown in the terminal output as xfailed if it fails, but if it unexpectedly passes then it will fail the test suite. This is particularly useful to mark functions that are always failing and there should be a clear indication if they unexpectedly start to pass (for example a new release of a library fixes a known bug).
            
            Defaults to xfail_strict, which is False by default.
        """)

    @pytest.mark.parametrize("test_input, expected", [("3+5", 8), ("2+4", 6), ("6*9", 42)])
    def test_parameterized(self, driver, logger):
        logger.info("""
            call a test function multiple times passing in different arguments in turn. 
            argvalues generally needs to be a list of values if argnames specifies only one name 
            or a list of tuples of values if argnames specifies multiple names. 
            
            Example: @parametrize('arg1', [1,2]) would lead to two calls of the decorated test function, 
            one with arg1=1 and another with arg1=2.
        """)

    @pytest.mark.usefixtures('driver', 'logger')
    def test_use_fixtures_example(self, driver, logger):
        logger.info("mark tests as needing all of the specified fixtures")

    @pytest.hookimpl(tryfirst=True)
    def test_tryfirst_hook(self, driver, logger):
        logger.info("mark a hook implementation function such that the plugin machinery will try to call it first/as "
                    "early as possible. DEPRECATED, use @pytest.hookimpl(tryfirst=True) instead.")

    @pytest.hookimpl(trylast=True)
    def test_trylast_hook(self, driver, logger):
        logger.info("mark a hook implementation function such that the plugin machinery will try to call it last/as "
                    "late as possible. DEPRECATED, use @pytest.hookimpl(trylast=True) instead.")







"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""

# Holds python command to run other commands.
# Options: python | python3
import os
import re
import platform
from nrobo.util.process import *


def verify_set_python_install_pip_command() -> None:
    """Verifies if python is installed on the host system.

    If found, sets the appropriate python command
    Else exit the nrobo framework with message.

    If found and sets python command, install pip as well."""

    from nrobo import EnvKeys, Environment, NROBO_CONST, Python
    # regular expression to verify python version
    # major.minor.nightly-build
    regx = re.compile(r'([\d]+).[\d]+.[\d]+.*')

    if regx.match(platform.python_version()) is None:
        print("Required dependency python is not installed on system! Please, install python >= 3.8 and retry.")
        exit(1)
    else:
        # python is installed on the host system

        if int(re.search(r'[\d]+', platform.python_version())[0]) >= 3:
            # check if python version is >=3

            python3 = "python3"

            if terminal([python3, "--version"], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT) == NROBO_CONST.SUCCESS:
                os.environ[EnvKeys.PYTHON] = python3
            elif terminal([os.environ[EnvKeys.PYTHON], "--version"], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT) == NROBO_CONST.SUCCESS:
                pass
            else:
                print("Required dependency python is not installed on system! Please, install python >= 3.8 and retry.")
                exit()

    # Install pip now!
    terminal([os.environ[EnvKeys.PYTHON], '-m', 'ensurepip', '--upgrade'], stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)



import pytest
from selenium.webdriver.common.by import By

from pages import Page


class TestNroboWaitMethods:

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_wait_for_page_to_be_loaded(self, driver, logger):
        """Example of wait for page to be loaded"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.wait_for_page_to_be_loaded()

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_wait_for_a_while(self, driver, logger):
        """Example of wait_for_a_while method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_wait_for_element_to_be_invisible(self, driver, logger):
        """Example of wait_for_element_to_be_invisible method"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkDisappearingElements = (By.CSS_SELECTOR, "[href='/disappearing_elements']")
        page.click(*lnkDisappearingElements)

        mnuPortfolio = (By.CSS_SELECTOR, "[href='/portfolio/']")
        eleMnuPortfolio = page.find_element(*mnuPortfolio)
        page.click(*mnuPortfolio)
        page.wait_for_element_to_be_invisible(eleMnuPortfolio)

        page.wait_for_a_while(3)






"""
=====================CAUTION=======================
DO NOT DELETE THIS FILE SINCE IT IS PART OF NROBO
FRAMEWORK AND IT MAY CHANGE IN THE FUTURE UPGRADES
OF NROBO FRAMEWORK. THUS, TO BE ABLE TO SAFELY UPGRADE
TO LATEST NROBO VERSION, PLEASE DO NOT DELETE THIS
FILE OR ALTER ITS LOCATION OR ALTER ITS CONTENT!!!
===================================================

@author: Panchdev Singh Chauhan
@email: erpanchdev@gmail.com
"""
import os
import subprocess


def terminal_nogui(command) -> int:
    """run command without any terminal output"""

    return terminal(command, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)


def get_os_system_command(command: [str]) -> str:
    """Returns os.system command from given <command>"""

    _command = ""

    for _cmd in command:
        _command += str(_cmd) + " "

    return _command


def terminal(command=[], stdin=None, input=None, stdout=None, stderr=None, capture_output=False, shell=False,
             cwd=None, timeout=None, check=False, encoding=None, errors=None, text=None, env=None,
             universal_newlines=None, debug=False, use_os_system_call=False) -> int:
    """Execute given command, command

    :param debug:
    :param universal_newlines:
    :param env:
    :param text:
    :param errors:
    :param encoding:
    :param check:
    :param timeout:
    :param cwd:
    :param shell:
    :param stderr:
    :param stdout:
    :param input:
    :param stdin:
    :param capture_output:
    :param command: command
    :return: status code
    """
    if use_os_system_call:
        return os.system(get_os_system_command(command))

    if debug is False:
        """check environment debug flag"""
        from nrobo import EnvKeys
        if str(os.environ[EnvKeys.DEBUG]) == "True":
            debug = True

    try:
        if text and capture_output:
            try:
                return subprocess.run(command, text=text, capture_output=capture_output)
            except subprocess.CalledProcessError as e:
                print(f"Command failed with return code {e.returncode}: \n{e}")
                return e.returncode

        if debug:
            try:
                subprocess.check_call(command)
            except subprocess.CalledProcessError as e:
                if e.returncode == 1:
                    print(f"Command failed with return code {e.returncode}: \n{e}")
                return e.returncode
        if (stdout and stderr) or debug is False:
            try:
                subprocess.check_call(command, stdout=subprocess.DEVNULL, stderr=subprocess.STDOUT)
            except subprocess.CalledProcessError as e:
                if e.returncode == 1:
                    print(f"Command failed with return code {e.returncode}: \n{e}")
                return e.returncode
        else:
            try:
                subprocess.check_call(command)
            except subprocess.CalledProcessError as e:
                if not e.returncode == 1:
                    print(f"Command failed with return code {e.returncode}: \n{e}")
                return e.returncode
    except FileNotFoundError as e:
        print(f"Command failed with FileNotFoundError!\n{e}")
        return 100
    except Exception as e:
        print(f"Command failed with exception:\n\t{e}")
        return 100

    return 0



import time

import pytest

from pages import Page

from selenium.webdriver.common.by import By


class TestNRoBoSeleniumWrapperMethods:

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_open_an_url(self, driver, logger):
        """Example to see how to open an URL"""

        page = Page(driver, logger)

        page.get("https://pypi.org/")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_browser_name(self, driver, logger):
        """Get driver/browser name"""

        page = Page(driver, logger)
        logger.info(f"Browser={page.name}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_title_of_the_current_page(self, driver, logger):
        """Get title of the current _page"""

        page = Page(driver, logger)
        page.get("http://pypi.org")
        logger.info(f"Title of the _page={page.title}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_url_of_the_current_page(self, driver, logger):
        """Get url of the current _page"""

        page = Page(driver, logger)
        page.get("http://pypi.org")
        logger.info(f"URL of the current _page={page.current_url}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_page_source_of_the_current_page(self, driver, logger):
        """Get _page source of the current _page"""

        page = Page(driver, logger)
        page.get("http://pypi.org")
        logger.info(f"Page source of the current _page\n\n{page.page_source}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_close_a_window_from_several_open_windows(self, driver, logger):
        """Example of closing a window"""

        page = Page(driver, logger)

        page.get("https://the-internet.herokuapp.com/")
        main_window = "The Internet"

        lnkMultipleWindow = (By.CSS_SELECTOR, "a[href='/windows']")
        page.click(*lnkMultipleWindow)

        lnkClickHere = (By.CSS_SELECTOR, "a[href='/windows/new']")
        page.click(*lnkClickHere)
        another_window = "New Window"

        # close the another window
        page.close(another_window)

        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_quit_all_windows(self, driver, logger):
        """Example of quiting all open windows"""

        page = Page(driver, logger)

        page.get("https://the-internet.herokuapp.com/")

        lnkMultipleWindow = (By.CSS_SELECTOR, "a[href='/windows']")
        page.click(*lnkMultipleWindow)

        lnkClickHere = (By.CSS_SELECTOR, "a[href='/windows/new']")
        page.click(*lnkClickHere)

        page.quit()

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_current_window_handle(self, driver, logger):
        """Get current window handle"""

        page = Page(driver, logger)

        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Current window handle={page.current_window_handle}")
        logger.info(f"Current window handle={page.windows['The Internet']}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_all_window_handles(self, driver, logger):
        """List out all the window handles"""

        page = Page(driver, logger)

        page.get("https://the-internet.herokuapp.com/")
        main_window_title = "The Internet"

        lnkMultipleWindow = (By.CSS_SELECTOR, "a[href='/windows']")
        page.click(*lnkMultipleWindow)

        lnkClickHere = (By.CSS_SELECTOR, "a[href='/windows/new']")
        page.click(*lnkClickHere)
        another_window_title = "New Window"

        logger.info(f"all handles = {page.window_handles}")
        logger.info(f"all handles = {page.windows}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_maximize_window(self, driver, logger):
        """Example of maximize window"""
        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")
        page.maximize_window()

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_fullscreen_mode(self, driver, logger):
        """Example of fullscreen mode"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")
        page.fullscreen_window()
        page.wait_for_a_while(5)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_minimize_window(self, driver, logger):
        """Example of minimize a window"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")
        page.minimize_window()
        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_print_page_and_save_as_pdf(self, driver, logger):
        """Print and save page as pdf"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")
        result = page.print_page()

        from nrobo.util.common import Common
        Common.save_as_pdf(result)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_switch_to_active_element(self, driver, logger):
        """Example of switch to element"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")
        result = page.switch_to_active_element()

        logger.info(f"Result={result}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_accept_alert(self, driver, logger):
        """accept alerts"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkJavaScriptAlert = (By.CSS_SELECTOR, "[href='/javascript_alerts']")
        page.click(*lnkJavaScriptAlert)

        btnJSAlert = (By.XPATH, "//button[text()='Click for JS Alert']")
        page.click(*btnJSAlert)
        page.accept_alert()

        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_working_with_frames(self, driver, logger):
        """Example of working with frames"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkFrames = (By.CSS_SELECTOR, "[href='/frames']")
        page.click(*lnkFrames)

        lnkNestedFrames = (By.CSS_SELECTOR, "[href='/nested_frames']")
        page.click(*lnkNestedFrames)
        logger.info(f"Title of the landing page={page.title}")

        frmTop = "frame-top"
        page.frame(frmTop)  # Switch to Top level frame

        frmLeft = "frame-left"
        page.frame("frame-left")  # Switch to Left frame now
        content = page.page_source
        logger.info(f"Left Frame Content\n{content}")

        # Switch back to default content that is to main body
        page.switch_to_default_content()
        logger.info(f"default content title={page.title}")

        frmTop = "frame-top"
        page.frame(frmTop)  # Switch again to Top level frame

        frmRight = "frame-right"
        page.frame(frmRight)  # Switch to Right frame now
        content = page.page_source
        logger.info(f"Right Frame Content\n{content}")

        # Switch back to default content using parent method
        page.switch_to_default_content()

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_switch_to_parent_frame(self, driver, logger):
        """Example of switch to parent frame"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkFrames = (By.CSS_SELECTOR, "[href='/frames']")
        page.click(*lnkFrames)

        lnkNestedFrames = (By.CSS_SELECTOR, "[href='/nested_frames']")
        page.click(*lnkNestedFrames)
        logger.info(f"Title of the landing page={page.title}")

        frmTop = "frame-top"
        page.frame(frmTop)  # Switch to Top level frame

        frmLeft = "frame-left"
        page.frame("frame-left")  # Switch to Left frame now
        content = page.page_source
        logger.info(f"Left Frame Content\n{content}")

        # Switch back to @pytest.mark.skip(reason="This test is an nRoBo example test.") default content that is to main body
        page.switch_to_parent_frame()
        logger.info(f"default content title={page.title}")

        frmRight = "frame-right"
        page.frame(frmRight)  # Switch to Right frame now
        content = page.page_source
        logger.info(f"Right Frame Content\n{content}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_switch_to_new_window(self, driver, logger):
        """Example of switch to new window"""

        page = Page(driver, logger)

        page.get("https://the-internet.herokuapp.com/")

        # open a new window as Tab
        page.switch_to_new_window("tab")
        page.get("http://google.com")
        page.wait_for_a_while(4)
        page.close("Google")

        # open a new window as New Window
        page.switch_to_new_window("window")
        page.get("http://google.com")
        page.wait_for_a_while(4)
        page.close("Google")

        logger.info(f"title={page.title}")
        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_browser_back_action(self, driver, logger):
        """Example of browser back action"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkFrames = (By.CSS_SELECTOR, "[href='/frames']")
        page.click(*lnkFrames)
        page.wait_for_a_while(2)

        # Perform browser back action
        page.back()
        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_browser_forward_action(self, driver, logger):
        """Example of browser forward action"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.get("http://google.com")

        # Perform browser back action
        page.back()
        page.wait_for_a_while(1)

        # Perform browser back action
        page.forward()
        page.wait_for_a_while(2)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_browser_refresh_action(self, driver, logger):
        """Example of browser refresh action"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        # perform browser refresh action
        page.refresh()
        page.wait_for_a_while(2)

        # once again
        page.refresh()
        page.wait_for_a_while(2)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_all_cookies(self, driver, logger):
        """Example of working with cookies

           get_cookies()

           get_cookie(name)

           delete_cookie(name)

           delete_all_cookies()

           add_cookie(cookies:{})"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        cookies = page.get_cookies()

        logger.info(f"All cookies= {cookies}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_add_cookie(self, driver, logger):
        """Example of adding a cookie"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        custom_cookies = {
            'name': 'company', 'value': 'ndi', 'path': '/'
        }
        page.add_cookie(custom_cookies)

        logger.info(f"updated cookies= {page.get_cookies()}")

        logger.info(f"Cookie value = {page.get_cookie('company')}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_delete_cookie(self, driver, logger):
        """Example of delete a cookie"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"All cookies ==> {page.get_cookies()}")

        page.delete_cookie('optimizelyEndUserId')

        logger.info(f"All cookies ==> {page.get_cookies()}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_delete_all_cookiee(self, driver, logger):
        """Example of delete all cookies"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"All cookies ==> {page.get_cookies()}")

        page.delete_all_cookies()

        logger.info(f"All cookies after deletion ==> {page.get_cookies()}")

    @pytest.mark.xfail
    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_implicitly_wait(self, driver, logger):
        """set implicitly wait"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.implicitly_wait(10)

        lnkArbitraryLink = (By.ID, "xxxhhhssjjsjjs")
        page.find_element(*lnkArbitraryLink)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_set_script_timeout(self, driver, logger):
        """set script timeout"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.set_script_timeout(30)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_set_page_load_timeout(self, driver, logger):
        """set page load timeout"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.set_page_load_timeout(30)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_all_timeouts(self, driver, logger):
        """get all timeouts"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Pageload timeout ===> {page.timeouts.page_load}")
        logger.info(f"Script timeout ===> {page.timeouts.script}")
        logger.info(f"Implicit timeout ===> {page.timeouts.implicit_wait}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_set_all_timeouts(self, driver, logger):
        """set all timeouts"""

        page = Page(driver, logger)
        page.timeouts.page_load = 1
        page.timeouts.script = 2
        page.timeouts.implicit_wait = 3

        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Page load timeout ===> {page.timeouts.page_load}")
        logger.info(f"Script timeout ===> {page.timeouts.script}")
        logger.info(f"Implicit timeout ===> {page.timeouts.implicit_wait}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_findelement(self, driver, logger):
        """example of findelement"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        lnkABTesting = (By.CSS_SELECTOR, '[href="/abtest"]')
        page.find_element(*lnkABTesting).click()
        page.wait_for_a_while(5)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_findelementssss(self, driver, logger):
        """example of findelementsss"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        allLinks = (By.CSS_SELECTOR, "ul li a")
        elements = page.find_elements(*allLinks)
        logger.info(f"Count of all links = {len(elements)}")
        page.wait_for_a_while(2)

        # click on 2nd link
        elements[1].click()
        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_capabilities(self, driver, logger):
        """example of capabilities"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Capabilities ===> {page.capabilities}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_screenshot_as_file(self, driver, logger):
        """example of get_screenshot_as_file"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.get_screenshot_as_file("screenshot.png")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_screenshot_as_png(self, driver, logger):
        """example of get_screenshot_as_png"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        bytes = page.get_screenshot_as_png()

        from nrobo.util.common import Common
        Common.save_bytes_to_file(bytes, "screenshot_as_png.png")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_screenshot_as_base64(self, driver, logger):
        """example of get_screenshot_as_base64"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        base64string = page.get_screenshot_as_base64()

        from nrobo.util.common import Common
        Common.save_base64string(base64string, "screenshot_as_base64string.png")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_set_window_size(self, driver, logger):
        """example of get_screenshot_as_base64"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.set_window_size(800, 400)
        page.wait_for_a_while(2)

        page.set_window_size(1200, 600)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_window_size(self, driver, logger):
        """example of get_window_size"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Current window size = {page.get_window_size()}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_set_window_position(self, driver, logger):
        """example of set_window_position"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.set_window_position(200, 300)
        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_set_window_rect(self, driver, logger):
        """example of set_window_rect"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        page.set_window_rect(200, 300, 600, 100)
        page.wait_for_a_while(3)

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_log_types(self, driver, logger):
        """example of log_types"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Orientation = {page.log_types}")

    @pytest.mark.skip(reason="This test is an nRoBo example test.")
    def test_get_log(self, driver, logger):
        """example of get_log"""

        page = Page(driver, logger)
        page.get("https://the-internet.herokuapp.com/")

        logger.info(f"Get Driver Log = {page.get_log('driver')}")
        logger.info(f"Get Driver Log = {page.get_log('browser')}")




